@@ -17,8 +17,16 @@ from PySide6.QtCore import (
     QTimer,
     Signal,
 )
-from PySide6.QtGui import QAction, QIcon
-from PySide6.QtWidgets import QApplication, QMainWindow, QSystemTrayIcon, QWidget
+from PySide6.QtGui import QAction, QIcon, QPixmap
+from PySide6.QtWidgets import (
+    QApplication,
+    QLabel,
+    QMainWindow,
+    QMenu,
+    QSplashScreen,
+    QStackedWidget,
+    QSystemTrayIcon,
+)
 from PySide6_GlobalHotkeys import Listener, bindHotkeys
 from voiceconversion.common.deviceManager.DeviceManager import DeviceManager
 from voiceconversion.downloader.WeightDownloader import (
@@ -36,6 +44,7 @@ from voiceconversion.VoiceChangerSettings import VoiceChangerSettings
 from voiceconversion.VoiceChangerV2 import VoiceChangerV2
 
 from .audio import Audio
+from .customizeui import CustomizeUiWidget
 from .exceptions import (
     FailedToSetModelDirException,
     PipelineNotInitializedException,
@@ -60,13 +69,34 @@ logger = logging.getLogger(__name__)
 # The IDs to talk with the keybindings configurator about the voice cards.
 VOICE_CARD_KEYBIND_ID_PREFIX = "voice_card_"
 
+ENABLE_PASS_THROUGH_KEYBIND_ID = "enable_pass_through"
+DISABLE_PASS_THROUGH_KEYBIND_ID = "disable_pass_through"
+
 
 class MainWindow(QMainWindow):
-    def __init__(self, modelDir: str, parent: QWidget | None = None):
-        super().__init__(parent)
+    def initialize(self, modelDir: str):
+        centralWidget = QStackedWidget()
+        self.setCentralWidget(centralWidget)
 
         self.windowAreaWidget = WindowAreaWidget(modelDir)
-        self.setCentralWidget(self.windowAreaWidget)
+        centralWidget.addWidget(self.windowAreaWidget)
+
+        self.customizeUiWidget = CustomizeUiWidget()
+
+        preferencesMenu = self.menuBar().addMenu("Preferences")
+
+        custumizeUiAction = QAction("Customize...", self)
+        custumizeUiAction.triggered.connect(
+            lambda: centralWidget.setCurrentWidget(self.customizeUiWidget)
+        )
+        self.customizeUiWidget.back.connect(
+            lambda: centralWidget.setCurrentWidget(self.windowAreaWidget)
+        )
+        centralWidget.addWidget(self.customizeUiWidget)
+
+        centralWidget.setCurrentWidget(self.windowAreaWidget)
+
+        preferencesMenu.addAction(custumizeUiAction)
 
         def onVoiceCardHotkey(shortcutId: str):
             if shortcutId.startswith(VOICE_CARD_KEYBIND_ID_PREFIX):
@@ -79,11 +109,15 @@ class MainWindow(QMainWindow):
                         and row >= 0
                     ):
                         self.windowAreaWidget.voiceCards.setCurrentRow(row)
+            elif shortcutId == ENABLE_PASS_THROUGH_KEYBIND_ID:
+                self.windowAreaWidget.passThroughButton.setChecked(True)
+            elif shortcutId == DISABLE_PASS_THROUGH_KEYBIND_ID:
+                self.windowAreaWidget.passThroughButton.setChecked(False)
 
         self.hotkeyListener = Listener()
         self.hotkeyListener.hotkeyPressed.connect(onVoiceCardHotkey)
 
-        configureKeybindingsAction = QAction("Configure Keybindings", self)
+        configureKeybindingsAction = QAction("Configure Keybindings...", self)
         configureKeybindingsAction.triggered.connect(
             lambda: bindHotkeys(
                 [
@@ -96,13 +130,37 @@ class MainWindow(QMainWindow):
                         self.windowAreaWidget.voiceCards.count(),  # 1 placeholder card
                         1,
                     )
+                ]
+                + [
+                    (
+                        ENABLE_PASS_THROUGH_KEYBIND_ID,
+                        {"description": "Enable Pass Through"},
+                    ),
+                    (
+                        DISABLE_PASS_THROUGH_KEYBIND_ID,
+                        {"description": "Disable Pass Through"},
+                    ),
                 ],
             )
         )
 
-        preferencesMenu = self.menuBar().addMenu("Preferences")
         preferencesMenu.addAction(configureKeybindingsAction)
 
+        self.systemTrayIcon = QSystemTrayIcon(self.windowIcon(), self)
+        systemTrayMenu = QMenu()
+        activateWindowAction = QAction("Show AVoc", self)
+        activateWindowAction.triggered.connect(
+            lambda: self.windowHandle().requestActivate()
+        )
+        quitAction = QAction("Quit AVoc", self)
+        quitAction.triggered.connect(lambda: QApplication.quit())
+        systemTrayMenu.addActions([activateWindowAction, configureKeybindingsAction])
+        systemTrayMenu.addSeparator()
+        systemTrayMenu.addAction(quitAction)
+        self.systemTrayIcon.setContextMenu(systemTrayMenu)
+        self.systemTrayIcon.setToolTip(self.windowTitle())
+        self.systemTrayIcon.show()
+
         self.vcm: VoiceChangerManager | None = (
             None  # TODO: remove the no-model-load CLI arg
         )
@@ -113,17 +171,14 @@ class MainWindow(QMainWindow):
         else:
             super().keyPressEvent(event)
 
-    def showTrayMessage(self):
-        systemTrayIcon = QSystemTrayIcon(self)
-        systemTrayIcon.show()
-        systemTrayIcon.showMessage(
-            "Title", "msg", QSystemTrayIcon.MessageIcon.Warning, 1000
-        )
+    def showTrayMessage(self, title: str, msg: str, icon: QIcon | QPixmap):
+        self.systemTrayIcon.showMessage(title, msg, icon, 1000)
 
 
 class VoiceChangerManager(QObject):
 
     modelUpdated = Signal(int)
+    modelSettingsLoaded = Signal(int, float, float)
 
     def __init__(self, modelDir: str, pretrainDir: str):
         super().__init__()
@@ -148,7 +203,6 @@ class VoiceChangerManager(QObject):
         )
 
         self.vc = VoiceChangerV2(voiceChangerSettings)
-        self.initialize()
 
     def getVoiceChangerSettings(self):
         voiceChangerSettings = VoiceChangerSettings()
@@ -205,11 +259,21 @@ class VoiceChangerManager(QObject):
             }
         )
 
+        self.modelSettingsLoaded.emit(
+            slotInfo.defaultTune,
+            slotInfo.defaultFormantShift,
+            slotInfo.defaultIndexRatio,
+        )
+
         if slotInfo.voiceChangerType == self.vc.get_type():
             self.vc.set_slot_info(
                 slotInfo,
                 self.pretrainDir,
             )
+            # TODO: unify changing properties that don't need reinit.
+            self.vc.vcmodel.settings.tran = slotInfo.defaultTune
+            self.vc.vcmodel.settings.formantShift = slotInfo.defaultFormantShift
+            self.vc.vcmodel.settings.indexRatio = slotInfo.defaultIndexRatio
         elif slotInfo.voiceChangerType == "RVC":
             logger.info("Loading RVC...")
             self.vc.initialize(
@@ -224,7 +288,32 @@ class VoiceChangerManager(QObject):
         else:
             logger.error(f"Unknown voice changer model: {slotInfo.voiceChangerType}")
 
-    def setRunning(self, running: bool):
+    def setModelSettings(
+        self,
+        pitch: int,
+        formantShift: float,
+        index: float,
+    ):
+        voiceChangerSettings = self.getVoiceChangerSettings()
+        val = voiceChangerSettings.modelSlotIndex
+        slotInfo = self.modelSlotManager.get_slot_info(val)
+        if slotInfo is None or slotInfo.voiceChangerType is None:
+            logger.warning(f"Model slot is not found {val}")
+            return
+
+        slotInfo.defaultTune = pitch
+        slotInfo.defaultFormantShift = formantShift
+        slotInfo.defaultIndexRatio = index
+
+        if self.vc.vcmodel is not None:
+            # TODO: unify changing properties that don't need reinit.
+            self.vc.vcmodel.settings.tran = slotInfo.defaultTune
+            self.vc.vcmodel.settings.formantShift = slotInfo.defaultFormantShift
+            self.vc.vcmodel.settings.indexRatio = slotInfo.defaultIndexRatio
+
+        self.modelSlotManager.save_model_slot(val, slotInfo)
+
+    def setRunning(self, running: bool, passThrough: bool):
         if (self.audio is not None) == running:
             return
 
@@ -239,6 +328,7 @@ class VoiceChangerManager(QObject):
                 voiceChangerSettings.serverReadChunkSize * 128,
                 self.changeVoice,
             )  # TODO: pass settings
+            self.audio.voiceChangerFilter.passThrough = passThrough
         else:
             self.audio = None
 
@@ -343,8 +433,10 @@ def main():
     app.setOrganizationName("AVocOrg")
     app.setApplicationName("AVoc")
 
+    iconFilePath = os.path.join(os.path.dirname(__file__), "AVoc.svg")
+
     icon = QIcon()
-    icon.addFile(os.path.join(os.path.dirname(__file__), "AVoc.svg"))
+    icon.addFile(iconFilePath)
 
     app.setWindowIcon(icon)
 
@@ -365,6 +457,15 @@ def main():
     timer.start(250)
     timer.timeout.connect(lambda: None)  # Let the interpreter run each 250 ms.
 
+    window = MainWindow()
+    window.setWindowTitle("AVoc")
+
+    splash = QSplashScreen(QPixmap(iconFilePath))
+    splash.show()  # Order is important.
+    window.show()  # Order is important. And calling window.show() is important.
+    window.hide()
+    app.processEvents()
+
     # Set the path where the voice models are stored and pretrained weights are loaded.
     appLocalDataLocation = QStandardPaths.writableLocation(
         QStandardPaths.StandardLocation.AppLocalDataLocation
@@ -372,37 +473,86 @@ def main():
     if appLocalDataLocation == "":
         raise FailedToSetModelDirException
 
+    # Check or download models that used internally by the algorithm.
     pretrainDir = os.path.join(appLocalDataLocation, PRETRAIN_DIR_NAME)
     asyncio.run(downloadWeight(pretrainDir))
 
-    window = MainWindow(os.path.join(appLocalDataLocation, MODEL_DIR_NAME))
-    window.setWindowTitle("AVoc")
+    # Lay out the window.
+    window.initialize(os.path.join(appLocalDataLocation, MODEL_DIR_NAME))
 
+    # Create the voice changer and connect it to the controls.
+    window.vcm = VoiceChangerManager(window.windowAreaWidget.modelDir, pretrainDir)
+    window.windowAreaWidget.startButton.toggled.connect(
+        lambda checked: window.vcm.setRunning(
+            checked, window.windowAreaWidget.passThroughButton.isChecked()
+        )
+    )
+
+    def setPassThrough(passThrough: bool):
+        if window.vcm.audio is not None:
+            window.vcm.audio.voiceChangerFilter.passThrough = passThrough
+
+    window.windowAreaWidget.passThroughButton.toggled.connect(setPassThrough)
+
+    modelSettingsGroupBox = window.windowAreaWidget.modelSettingsGroupBox
+
+    def onModelSettingsChanged():
+        window.vcm.setModelSettings(
+            pitch=modelSettingsGroupBox.pitchSpinBox.value(),
+            formantShift=modelSettingsGroupBox.formantShiftDoubleSpinBox.value(),
+            index=modelSettingsGroupBox.indexDoubleSpinBox.value(),
+        )
+
+    modelSettingsGroupBox.changed.connect(onModelSettingsChanged)
+
+    interfaceSettings = QSettings()
+    interfaceSettings.beginGroup("Interface")
+
+    def onVoiceCardChanged() -> None:
+        modelSettingsGroupBox.changed.disconnect(onModelSettingsChanged)
+        window.vcm.initialize()
+        modelSettingsGroupBox.changed.connect(onModelSettingsChanged)
+        if bool(interfaceSettings.value("showNotifications", True)):
+            voiceCardWidget: QLabel = window.windowAreaWidget.voiceCards.itemWidget(
+                window.windowAreaWidget.voiceCards.currentItem()
+            )
+            pixmap = voiceCardWidget.pixmap()
+            window.showTrayMessage(
+                window.windowTitle(),
+                f"Switched to {voiceCardWidget.toolTip()}",
+                pixmap,
+            )
+
+    def onModelSettingsLoaded(pitch: int, formantShift: float, index: float):
+        modelSettingsGroupBox.pitchSpinBox.setValue(pitch)
+        modelSettingsGroupBox.formantShiftDoubleSpinBox.setValue(formantShift)
+        modelSettingsGroupBox.indexDoubleSpinBox.setValue(index)
+
+    window.vcm.modelSettingsLoaded.connect(onModelSettingsLoaded)
+
+    window.windowAreaWidget.voiceCards.currentRowChanged.connect(onVoiceCardChanged)
+    window.windowAreaWidget.voiceCards.droppedModelFiles.connect(
+        lambda loadModelParams: window.vcm.importModel(loadModelParams)
+    )
+    window.windowAreaWidget.voiceCards.droppedIconFile.connect(
+        lambda slot, iconFile: window.vcm.setModelIcon(slot, iconFile)
+    )
+    window.vcm.modelUpdated.connect(
+        lambda slot: window.windowAreaWidget.voiceCards.onVoiceCardUpdated(slot),
+    )
+
+    audioSettingsGroupBox = window.windowAreaWidget.audioSettingsGroupBox
+    audioSettingsGroupBox.sampleRateComboBox.currentIndexChanged.connect(
+        lambda: window.vcm.initialize()
+    )  # It isn't running when changing sample rate.
+
+    # Load the current voice model if any.
     if not clParser.isSet(noModelLoadOption):
-        window.vcm = VoiceChangerManager(window.windowAreaWidget.modelDir, pretrainDir)
-        window.windowAreaWidget.startButton.toggled.connect(
-            lambda checked: window.vcm.setRunning(checked)
-        )
-        window.windowAreaWidget.voiceCards.currentRowChanged.connect(
-            lambda: window.vcm.initialize()
-        )
-        window.windowAreaWidget.voiceCards.droppedModelFiles.connect(
-            lambda loadModelParams: window.vcm.importModel(loadModelParams)
-        )
-        window.windowAreaWidget.voiceCards.droppedIconFile.connect(
-            lambda slot, iconFile: window.vcm.setModelIcon(slot, iconFile)
-        )
-        window.vcm.modelUpdated.connect(
-            lambda slot: window.windowAreaWidget.voiceCards.onVoiceCardUpdated(slot),
-        )
-        (
-            (
-                window.windowAreaWidget.audioSettingsGroupBox.sampleRateComboBox
-                # Lots of parenthesis because black code formatter doesn't break lines.
-            ).currentIndexChanged.connect(lambda: window.vcm.initialize())
-        )  # It isn't running when changing sample rate.
+        window.vcm.initialize()
 
+    # Show the window
     window.resize(1980, 1080)  # TODO: store interface dimensions
     window.show()
+    splash.finish(window)
 
     sys.exit(app.exec())
